#include <stdio.h>
#include "misc.h"

void Distrib (double *s, int N, double *h, int *d, int PO)
// input:  s - postupnost hodnot, N pocet hodnot v postupnosti s,
// PO - pocet bodov mriezky
// output: h - body mriezky, d - zodpovedajuci pocet hodnot 
{
    int i;

    for (i=0; i<=PO; ++i) d[i]=0;

    double min = MAXDOUBLE, max = 0.0;
    for (i=0; i<N; ++i)
	if (s[i]>max) max=s[i]; else if (s[i]<min) min=s[i];

    min = 0.117;
    max = 0.125;

    double S = (max-min)/PO;
    for (i=0; i<N; ++i) ++d[(int) rint((s[i]-min)/S)];
    for (i=0; i<=PO; ++i) h[i] = min+i*S;
}

void Gen_Noise (double *s, int N)
// vrati retazec s, obsahujuci N hodnot podla Gaussovho rozdelenia
{
    double r1, r2, rsq;

    int i = 0;
    while (i<N)
    {
	rsq = MAXDOUBLE;
	while ((rsq>=1.0)||(rsq==0.0))
	{
	    r1  = 2.0*((double)rand()/RAND_MAX)-1.0;
	    r2  = 2.0*((double)rand()/RAND_MAX)-1.0;
	    rsq = pow(r1,2.0) + pow(r2,2.0);		
	}
	rsq = sqrt(-2.0*log(rsq)/rsq);
	s[i++] = r1*rsq;
	s[i++] = r2*rsq;
    }
}

void Sort (double *s, int N)
{
    int i, j, k;
    int    *p = new (int)[N];
    double *d = new (double)[N];

    for (i=0; i<N; ++i) p[i]=0;
    for (i=0; i<N; ++i)
	for (j=i+1; j<N; ++j)
	{
	   k = (s[i]>=s[j]) ? i : j; 
	   ++p[k];
	}	
    for (i=0; i<N; ++i) d[p[i]] = s[i]; 
    for (i=0; i<N; ++i) s[i] = d[i];

    return;
}

/*
void Make_Film (char nazov[])
{
    int t, i;
    double x, y, z;
    char trash[300];
    FILE *source = fopen(nazov, "r");
    FILE *target;
    
    printf("set noborder\n");
    printf("set noxtics\n");
    printf("set noytics\n");
    printf("set noztics\n");

    for (t=1; t<=50000; ++t)
    {

	if (t%1) continue;

	sprintf(trash, "time__%d", t);
	target = fopen(trash,"w");

	printf("splot \[-1.5:1.5]\[-1.5:1.5]\[-1.5:1.5] \"%s\" u 1:2:3 w linesp\n", trash);
	fflush(stdout);

	fgets(trash, 300, source);
	for (i=0; i<300; ++i)
	{
	    fscanf(source,"%lg%lg%lg", &x, &y, &z);
	    fprintf(target,"%g %g %g\n", x, y, z);
	    if (i%2) fprintf(target,"\n\n");
	    fgets(trash, 300, source);
	}
	fgets(trash, 300, source);

	fclose(target);
    } 		
    fclose(source);

    exit(0);
}
*/

void Make_Film (char nazov[])
{
    int t, i;
    double x1, y1, z1, x2, y2, z2, dist;
    char trash[300];
    FILE *source = fopen(nazov, "r");
    
    for (t=1; t<=50000; ++t)
    {
	fgets(trash, 300, source);
	for (i=0; i<150; ++i)
	{
	    fscanf(source,"%lg%lg%lg", &x1, &y1, &z1);
	    fgets(trash, 300, source);
	    fscanf(source,"%lg%lg%lg", &x2, &y2, &z2);
	    fgets(trash, 300, source);

	    if (i==1)
	    {
		dist = sqrt(pow(x1-x2,2.0)+pow(y1-y2,2.0)+pow(z1-z2,2.0));
		printf("%d    %g\n", t, dist); fflush(stdout);
	    }
	}
	fgets(trash, 300, source);
    } 		
    fclose(source);
    exit(0);
}

inline double Min (double *d, int N)
{
    int i;
    double min = MAXDOUBLE;
    for (i=0; i<N; ++i) if (d[i]<min) min=d[i];
}

inline double Max (double *d, int N)
{
    int i;
    double max = -MAXDOUBLE;
    for (i=0; i<N; ++i)	if (d[i]>max) max=d[i];
}

void WriteDistrib (double *d, int N, double min, double max, int I)
{
    double step = (max-min)/I;

    int    *n = new (int)[I+1];
    double *s = new (double)[I+1];

    int i;
    for (i=0; i<=I; ++i)
    {
	s[i] = min + i*step;
	n[i] = 0;
    }

    int index;
    for (i=0; i<N; ++i)
    {
	index = (int) rint((d[i]-min)/step);
	if ((index>=0)&&(index<=I)) ++n[index];
    }

    for (i=0; i<=I; ++i) printf("%g     %g\n", s[i], n[i]/(step*I*N));
}

void WriteDistrib (double *d, int N, int I)
{
    double min = MAXDOUBLE;
    double max = -MAXDOUBLE;

    int i, j;

    for (i=0; i<N; ++i)
    {
	if (d[i]<min) min=d[i];
	if (d[i]>max) max=d[i];
    }

    WriteDistrib (d, N, min, max, I);
}
