#include <stdio.h>
#include <math.h>
#include <assert.h>
#include <string.h>
#include <time.h>

#define DIST 28800

#define SMOOTH

#define BIGSET

//#define CHECK

#ifdef BIGSET
#define TRAJECTORY     "big_init_file"
#define NO_ATOMS	2400
#define NO_MOLECULES	1200
#define BOXL		3.73643
#else
//#define TRAJECTORY    "equilib10ps"
//#define TRAJECTORY    "npinit"
#define TRAJECTORY    "initconf"
//#define TRAJECTORY      "small_init_file"
#define NO_ATOMS	300
#define NO_MOLECULES	150
#define BOXL		1.868
//#define BOXL		1.765
#endif

#define NO_TIME_SLICES	2
#define ATOMS_MOLECULE	2
#define DELTAT		0.001
#define TS_LENGTH	1020
#define VACF_LENGTH	5000
#define NOISE_POINTS    (5*NO_ATOMS)

#define PRED_COEFF_SETS 100

#define PREDICT 2
#define CALCUL  1
#define UPDATE  0

#define PI      3.141592654
#define EPS	0.512167
#define EPS48   (48.0*EPS)
#define SIGMA	29.5e-2
#define SIGSQ   (SIGMA*SIGMA)
#define FC	70.8805e+4
#define L0	12.0741e-2 
#define RCUT	0.84
#define RCUTSQ  (RCUT*RCUT)
#define SW	0.8
#define SWSQ	(SW*SW)
#define VERLET  0.0005
#define CALCLIM 0.0
#define CLSQ    (CALCLIM*CALCLIM)
#define TEMP	73.0
#define MASS	1599.9304e-2
#define BOLTZ	8.3144e-3
#define GAMMA   0.99

class type_atom
{
private:
	double *rold, *r, *ib, *rs;
	double *vold, *v;
	double *f, *fb;
	double **fnb;
	double **ts;
	double **vacf;
	double rijsq, *rij;
	double ***der;

	int PRED, p;
	
	double R13_3R12R0;
	double R1R03;
	
	inline double smooth (double r)
	{ return (R13_3R12R0+6*RCUT*SW*r-3*(RCUT+SW)*r*r+2*r*r*r)/R1R03;}

	friend class type_liquid;
public:
	type_atom();
	void Leap();
	double from (type_atom & A);
        void SaveVelocities();
	void WriteForces(int pocet);
	void ComputeDerivations(int PRED_ORDER);
	double Taylor (int k, int PRED_ORDER, double dt);	
	void operator= (type_atom & A);    // copy atom
	void operator<= (FILE *source);	   // load atom
	void operator()();		   // put atom into box
	void operator()(int i);		   // put atom into box
	double operator- (type_atom & A);  // get bonding interaction	
	double operator+ (type_atom & A);  // get non-bonding to time series
	double operator* (type_atom & A);  // get non-bonding outside ts

	void operator<= (int k)
	{
	    int p;

	    fnb[k] = &ts[k][PRED];
	    for (p=0; p<PRED; ++p) ts[k][p] = ts[k][TS_LENGTH-PRED+p];
	}

	void operator>(FILE *target)
	{
	    int k;
				    
	    for (k=500; k<500+100; ++k) fprintf(target,"%.10g\t  %.10g\t  %.10g\n",
	    	ts[0][k], ts[1][k], ts[2][k]);
	}
};

class type_liquid
{
private:
	int aktual;

	int curr_pos_vacf;

	double KBTDM;
	double FAC;
	double VK;
	double VLJ;
	double VB;
	double VP;

	double VAR;

	double *wn;
	double *wn2;

	int NO_SHELLS;
	int zero;

	int incalc;
	unsigned long *calcul;

	int *inshell;
	unsigned long **shell;

	int incount;
	unsigned long *inside;

	int inbuffer;
	unsigned long *buffer;

	int *pics;
	unsigned long *cs;

	inline double Predicted (double *b)
	{
	    double tmp = 0.0;
	    for (p=1; p<=PRED; ++p) tmp += a[p]*b[-p];
	    return (tmp);
	}


	friend class type_molecule;
public:
	double vd[DIST];

	type_atom *atom;

	int p, PRED;
	double *a;

        type_liquid (int MDrun);
	void LeapFrog (int mode);
	void CalculateForces (int mode);
	void Set_Predictor_Weights (int NE, int PRED_ORDER, int MAX);
	void Set_Predictor_Weights (int P, double *x);
	void Set_Taylor_Weights (int NE, int PRED_ORDER, int MAX);
	void Set_Papoulis_Weights (int NE, int PRED_ORDER, int MAX);
	void SavePositions ();
	void WriteForces (int pocet);
	void ComputeDerivations(int PRED);
	double MSD();
	void Scale ();
	void Write_RDF (double min, double max, int I);
	void Write_VACF(int end);
	void Write2File (FILE *target);

	double operator () (int i, int k) {return(*atom[i].fnb[k]);};

	void operator= (type_liquid & A)
	{
	    KBTDM = A.KBTDM;
	    FAC   = A.FAC;	    
	    VK    = A.VK;
	    VLJ   = A.VLJ;
	    VB    = A.VB;
	    VP    = A.VP;


	    int i, j;

	    for (i=0; i<NO_ATOMS; ++i) atom[i]=A.atom[i];

	    NO_SHELLS = A.NO_SHELLS;
	    zero      = A.zero;

	    incalc    = A.incalc;
	    incount   = A.incount;
	    inbuffer  = A.inbuffer;
	    
	    for (i=0; i<incalc; ++i)   calcul[i] = A.calcul[i];
	    for (i=0; i<incount; ++i)  inside[i] = A.inside[i];	    
	    for (i=0; i<inbuffer; ++i) buffer[i] = A.buffer[i];
	    
	    for (i=0; i<NO_SHELLS; ++i)
	    {
	    	inshell[i] = A.inshell[i];
	        for (j=0; j<inshell[i]; ++j) shell[i][j] = A.shell[i][j];
	    }

	    if (A.cs==A.buffer)
	    {
		cs = buffer;	    
		pics = &inbuffer;
	    }
	    else
	    {
		cs = shell[(zero+1)%NO_SHELLS];
		pics = &inshell[(zero+1)%NO_SHELLS];
	    }
	}

	double Ekin() {return VK;};
	double Epot() {return VP;};
	double Ebond(){return VB;};
	double ETOT;
};
