#include <stdio.h>
#include "svd.h"

extern "C"
{
    extern void dgesvd_(char *jobu, char *jobvt, int *m, int *n, double *a,
                int *lda, double *s, double *u, int *ldu, double *vt,
                int *ldvt, double *work, int *lwork, int *info);
}

void SVD (double **e, int m0, int ne, double *d, double *x)
// double **e		(in)	matrix of inputs 
// int      m0		(in) 	length of the segment (i. e. PRED_ORDER) = number of columns of e
// int      ne		(in)	number of inputs = number of lines of e
// int     *d		(in)	vector of "desired" outputs (dimension = ne)
// double  *x		(out)	predictor coefficients (dimension = m0)
{
    int i, j, l;

    double *a = new (double)[ne*m0];

    l=-1;
    for (j=0; j<m0; ++j)
	for (i=0; i<ne; ++i) a[++l] = e[i][j];

    double *u  = new (double)[ne*m0];
    double *vt = new (double)[m0*m0];
    double *w  = new (double)[m0];

    char jobu  = 'S';
    char jobvt = 'A';
    int m      =  ne;
    int n      =  m0;
    int lda    =  ne;
    int ldu    =  ne;
    int ldvt   =  m0;

    int min   = (m>=n) ? n : m;
    int max   = (m>=n) ? m : n;
    int lwork = ((3*min+max)>=(5*min)) ? 3*min+max : 5*min;
    int info;

    double *work = new (double)[lwork*lwork];
    dgesvd_(&jobu,&jobvt,&m,&n,a,&lda,w,u,&ldu,vt,&ldvt,work,&lwork,&info);
    delete(work);

    l = -1;
    double **U  = new (double *)[ne];
    for (i=0; i<ne; ++i)  U[i] = new (double)[m0];

    for (j=0; j<m0; ++j)
	for (i=0; i<ne; ++i)
	    U[i][j] = u[++l];

    l = -1;
    double **V = new (double *)[m0];
    for (i=0; i<m0; ++i)
    {
	V[i] = new (double)[m0];
	for (j=0; j<m0; ++j) V[i][j] = vt[++l];
    }

    double *tmp = new (double)[m0];
    for (j=0; j<m0; ++j)
    {
	tmp[j] = 0.0;
	if (w[j])
	{
	    for (i=0; i<ne; ++i) tmp[j] += U[i][j]*d[i];
	    tmp[j] /= w[j];
	}
    }

    for (j=0; j<m0; ++j)
    {
	x[j] = 0.0;
	for (l=0; l<m0; ++l) x[j] += V[j][l]*tmp[l];
    }
    delete(tmp);

    for (i=0; i<ne; ++i) delete(U[i]);
    delete(U);

    for (i=0; i<m0; ++i) delete(V[i]);
    delete(V);

    delete(u);
    delete(vt);
    delete(w);
    delete(a);
}
