//posunut atom do kocky

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#define DIM     3
#define NATOMS  300
#define ZDROJ   "oxygen"
#define DELTAT  0.001

#define FC	70.8805e+4      // fyz. konstanta harmon. pot.
#define L0	12.0741e-2      // fyz. konstanta harmon. pot.

#define BOXSIZE 1.868           // velkost boxu

class EucleidDistance {
	public:
		double getDistance(double *p1, double *p2) {
			double result;
			result = sqrt(
				pow(p2[0] - p1[0], 2) +
				pow(p2[1] - p1[1], 2) +
				pow(p2[2] - p1[2], 2)
			);
			return result;
		}
};



class Atom
{
private:
	double *r;	// suradnice atomu (3D)
	double *v;	// rychlosti
	double *f;	// sily posobiace na atom

	double *r_inbox; //suradnice vo vnutri kocky
public:
	Atom();	// konstruktor - deklaracia
	void operator<(FILE *source);
	void operator>(FILE *target) {
	  fprintf(target,"\\vec{v}: %.10g  %.10g  %.10g\n", v[0], v[1], v[2]);
	}

	double *getR() {
		return r;
	}

	void setR(double *r_new) {
		for(int i = 0; i < DIM; i++) {
			r_new[i] = r[i];
		}
	}
};

Atom::Atom()	// konstruktor - definicia
{
	r = new (double)[DIM];
	v = new (double)[DIM];
	f = new (double)[DIM];
}

void Atom::operator<(FILE *source)	// pretazenie operatora <
{
    int k;
    double *tmp;
    double *r_old = new (double)[DIM];

    tmp  = r_old;
    r_old = r;
    r    = tmp;

    fscanf(source, "%lg%lg%lg%", &r[0], &r[1], &r[2]);

    for (k=0; k<DIM; ++k)
      v[k] = (r[k]-r_old[k])/DELTAT;

    delete (r_old);
}
//------------------------------------------------------------------------
// TYPE SYSTEM
//------------------------------------------------------------------------
class type_system	// konstruktor
{
private:
	Atom *atom;

public:
        type_system(char *startfrom);
	double* wrap(double* d);
};

double* type_system::wrap(double *d) {
  double* result = new double[DIM];

  for(int i = 0; i < DIM; i++) {
    int sgn = d[i] > 0 ? 1 : -1;
    result[i] = d[i] - (sgn * BOXSIZE);
    if(d[i] != result[i])
      printf("Wrap: %.10g->%.10g\n", d[i], result[i]);
  }
  return(result);
}

type_system::type_system(char *startfrom)
{
	atom = new (Atom)[NATOMS];

	FILE *source = fopen(startfrom,"r");

	int i;
	//nacitaj prvych NATOMS v case t=0
	for (i=0; i<NATOMS; ++i)
	  atom[i]<source;	// cas t=0
	//zvysnych NATOMS v case t=\delta t
	for (i=0; i<NATOMS; ++i)
	  atom[i]<source;	// cas t=1

	fclose(source);

	double *dist = new double[NATOMS / 2];

	//vypocet vzdialenosti
        for (i=0; i<NATOMS/2; i += 2) {
          EucleidDistance *d = new EucleidDistance();
	  dist[i] = d->getDistance(wrap(atom[i].getR()), wrap(atom[i+1].getR()));
	  delete(d);
	  printf("%d -- %d: %.10g\n", i, i+1, dist[i]);
	}

	//vypocet sily viazanych
	for(i = 0; i < NATOMS/2; i += 2) {
	  double sila = FC * (1 - (L0 / dist[i]));
	  printf("Sila viazanych %d - %d: %.10g\n", i, i+1, sila);
	}



        delete(dist);

	/*
	for(i = 0; i < NATOMS; i++) {
		EucleidDistance::getDistance(atom[i].getR(), atom[i+NATOMS].getR());
	}
	*/

}

type_system O2(ZDROJ);

/*
class AtomWrapper {
	public:
		void wrap(Atom &atom) {
			double *dist = new dist[DIM]
			for(int i = 0; i < DIM; i++) {
				//posunieme o niekolko celociselnych nasobkov do vnutra kocky
				double r = *(atom->r[i]);

				atom->setR()[i] = r - rint(r / BOXSIZE);
			}
		}

}
*/


main(int argc, char *argv[]) {

}
